#!/usr/bin/env python

# The Expat License
#
# Copyright (c) 2019, Shlomi Fish
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

# from functools import reduce
import functools
import pprint
import sys

# import numpy as np

# import six
from six import print_

import sparse_list


np = False
DEBUG = 0
pprint.pprint("")


def digit_sum(n):
    return sum([int(d) for d in str(n)])


def brute_find_lowest_num_with_digit_sum(n):
    ret = 1
    while True:
        if digit_sum(ret) == n:
            return ret
        ret += 1


def brute_calc_f(n, m):
    ret = 1
    while True:
        if digit_sum(ret) == n:
            m -= 1
            if not m:
                return ret
        ret += 1


def not_so_brute_calc_f(n, m):
    def myiter(L, prefix, digsum):
        if len(prefix) == L:
            if digsum == n:
                yield prefix
            return
        if digsum > n:
            return
        top = digsum + (L-len(prefix))*9
        if top < n:
            return
        for d in range((len(prefix) == 0), 10):
            for x in myiter(L, prefix+[d], digsum+d):
                yield x

    def l_iter():
        L = 0
        while True:
            it = myiter(L, [], 0)
            for x in it:
                yield x
            L += 1
    it = l_iter()
    mm = m
    for x in it:
        mm -= 1
        if mm == 0:
            return int("".join([str(y) for y in x]))


def brute_count(n, bottom, top):
    r = 0
    for x in range(int(bottom), int(top)):
        if digit_sum(x) == n:
            r += 1
    return r


def mod_matmul(MOD, n, m1, m2):
    ret = np.zeros((n, n), dtype=np.int64)
    for y in range(n):
        row = m1[y]
        for x in range(n):
            ret[y, x] = sum(int(row[i])*int(m2[i, x]) for i in
                            range(n)) % MOD
    return ret

# Planning:
# Row 0 - the constant 1
# Row 1 - "1, 19, 199, 1999" ;
# "2, 29, 299, 2999" / etc.
# Row 2 - the sum


def compare(have, want, blurb):
    print_('blurb =', blurb, 'have =', have, 'want =', want)
    assert have == want


@functools.lru_cache(maxsize=10000)
def nCr(n, k):
    if k > (n >> 1):
        return nCr(n, n-k)
    r = 1
    for x in range(n-k + 1, n+1):
        r *= x
    d = 1
    for x in range(2, k+1):
        d *= x
    # assert r % d == 0
    r //= d
    # compare(r, FACTS[n] // FACTS[k] // FACTS[n-k],
    #         "ncr(k={};n={})".format(k, n))
    return r


ctr = 0
count9 = 0
non9 = 0


class MatrixMod:
    DIGITS_CYCLE = 9

    def __init__(self, MOD):
        self.MOD = MOD
        self.m1 = self.gen_matrix()
        self.range9 = range(self.DIGITS_CYCLE)

    def gen_matrix(self):
        L = 3
        return 0
        ret = np.zeros((L, L), dtype=np.int64)

        def assign(from_, to, val):
            ret[to, from_] = val

        assign(0, 0, 1)
        assign(0, 1, 9)
        assign(1, 1, 10)
        assign(0, 2, 9)
        assign(1, 2, 10)
        assign(2, 2, 1)

        return ret

    def mass_check(self):
        range_sum = 0
        for x in range(1, 1000):
            term = brute_find_lowest_num_with_digit_sum(x)
            print_("s({}) = {}".format(x, term),
                   flush=True)
            range_sum += term
            compare(
                self.calc_s(x),
                range_sum % self.MOD, "calc_s[{}]".format(x)
            )

    def calc_s(self, n):
        return sum(
            [self.calc_s_for_uppermost_digit(uppermost_digit, n)
             for uppermost_digit in range(self.DIGITS_CYCLE)]
        ) % self.MOD

    # We calculate the sums for:
    #   [uppermost_digit = 0]: 9 + 99 + 999 + 9999 ...
    #   [uppermost_digit = 1]: 1 + 19 + 199 + 1999 + 19999 ...
    #   2: 2 + 29 + 299 + 2999 + 29999 ...
    # Etc. separately.
    def calc_s_for_uppermost_digit(self, uppermost_digit, n):
        n_mod = n
        while n_mod % self.DIGITS_CYCLE != uppermost_digit:
            n_mod -= 1
        e = n_mod // self.DIGITS_CYCLE
        if e < 0:
            return 0
        if e == 0:
            return uppermost_digit
        mat = self.mat_exp_mod(e)
        return (mat[2, 0] + (int(mat[2, 1]) + int(mat[2, 2]))
                * uppermost_digit)

    def mat_exp_mod(self, e):
        m1 = self.m1
        if e == 1:
            return m1
        sub = self.mat_exp_mod(e >> 1)
        ret = (np.matmul(sub, sub) % self.MOD)
        return (np.matmul(ret, m1) % self.MOD) if e & 1 else ret

    def _gen_01_L_initial_sequences(self, L, digit_sum):
        seqs = [0 for _ in range(L+1)]

        def rec(so_far, rem):
            ll = len(so_far)
            if ll > L:
                return
            padded = rem + 9 * (L - ll)
            if padded < digit_sum:
                return
            if rem > digit_sum:
                return
            is_valid = padded == digit_sum
            if is_valid:
                seqs[ll] += 1
            for i in self.range9:
                rec(so_far + [i], rem + i)

        rec([], 0)
        return seqs

    def _gen_L_initial_sequences(self, L, digit_sum):
        sequences1 = [[] for _ in range(L+1)]
        sequences0 = [[] for _ in range(L+1)]

        def rec(so_far, rem):
            ll = len(so_far)
            if ll > L:
                return
            padded = rem + 9 * (L - ll)
            if padded < digit_sum:
                return
            if rem > digit_sum:
                return
            is_valid = padded == digit_sum
            if is_valid:
                if ll and so_far[0] == 0:
                    sequences0[ll].append(so_far)
                else:
                    sequences1[ll].append(so_far)
            for i in self.range9:
                rec(so_far + [i], rem + i)

        rec([], 0)
        seqs = (sequences0, sequences1)
        return seqs

    def calc_f(self, n, m):
        remainder = n % self.DIGITS_CYCLE
        count9s = n // self.DIGITS_CYCLE
        L = count9s+1 if remainder else count9s
        remaining_m = m
        if m == 1:
            return int(str(remainder) + "9"*count9s)
        while remaining_m:
            if remainder == 0:
                remainder += self.DIGITS_CYCLE
                count9s -= 1
            seqs = self._gen_L_initial_sequences(L, n)
            # print_("seqs0={} seqs1={}".format(sequences0, sequences1))
            total_diff = 0
            for leading_digit in [0, 1]:
                digit_l = L - 1 + leading_digit
                s = seqs[leading_digit]
                c = 0
                for seq_len in range(digit_l + 1):
                    count = len(s[seq_len])
                    if count:
                        total_diff += count * nCr(digit_l, seq_len)
                        if c == 1:
                            c = 2
                    else:
                        if not c:
                            c = 1
                        elif c == 2:
                            # compare(seq_len, 0, 'seq_len')
                            break
            newfound = remaining_m - total_diff
            if newfound > 0:
                remaining_m = newfound
                L += 1
            else:
                ret = self._calc_f_finetune(L, seqs, n, remaining_m)
                self._nullify_seq()
                return ret

    def _nullify_seq(self):
        self.zs = None
        self._seq_counts = None

    def _set_seq(self, val):
        self.zs = val
        self._seq_counts = \
            [sum(len(x) for x in self.zs[i]) for i in self.nrange]

    def _calc_f_finetune(self, L, seqs, n, remaining_m):
        global count9
        global non9

        def _get_initial_by_digit():
            return [[] for _ in range(10)]
        prefix = sparse_list.SparseList(0, "9")
        seqs_incremental = []
        ncrl = L-1
        self.nrange = range(ncrl+1)
        cant_start_with_zero = 1
        zrange = range(cant_start_with_zero, 9)
        digsum = 0

        for s in seqs:
            dest_seq_by_len = []
            for ll, x in enumerate(s):
                if ll == 0:
                    continue
                dest_len_seq__by_digit = _get_initial_by_digit()
                for y in x:
                    if len(y):
                        dest_len_seq__by_digit[y[0]].append(y[1:])
                dest_seq_by_len.append(tuple(dest_len_seq__by_digit))
            seqs_incremental.append(tuple(dest_seq_by_len))
        self._set_seq(seqs_incremental[1])

        def descend_several_9s(count_new_9s):
            nonlocal digsum
            nonlocal ncrl
            prefix.size += count_new_9s
            digsum += count_new_9s * 9
            ncrl -= count_new_9s
            self.nrange = range(ncrl+1)

        def descend(d):
            nonlocal cant_start_with_zero
            nonlocal digsum
            nonlocal ncrl
            if cant_start_with_zero:
                cant_start_with_zero = 0
                nonlocal zrange
                zrange = self.range9
                dest_seq_by_len = []
                for ll in range(len(seqs_incremental[0])):
                    dest_len_seq__by_digit = _get_initial_by_digit()
                    for s in seqs_incremental:
                        x = s[ll]
                        # compare(len(x), 10, 'plp')
                        for dd, y in enumerate(x):
                            # compare(len(y), 10, 'plp')
                            dest_len_seq__by_digit[dd] += y
                    dest_seq_by_len.append(tuple(dest_len_seq__by_digit))
                self._set_seq(dest_seq_by_len)
            prefix.append(str(d))
            digsum += d
            ncrl -= 1
            self.nrange = range(ncrl+1)
            # pprint.pprint([prefix.elements, prefix.size])
            if d < 9:
                dest_seq_by_len = []
                for ll in range(1, len(self.zs)):
                    dest_len_seq__by_digit = _get_initial_by_digit()
                    x = self.zs[ll]
                    for y in x[d]:
                        if (len(y)):
                            dest_len_seq__by_digit[y[0]].append(y[1:])
                    dest_seq_by_len.append(tuple(dest_len_seq__by_digit))
                self._set_seq(dest_seq_by_len)

        while True:
            # pprint.pprint(locals())
            def count_perms(ll, lenitem):

                def _c():
                    if lenitem == 0:
                        return 1
                    if lenitem == 1:
                        return ncrl
                    return nCr(ncrl, lenitem)
                if ll:
                    return _c() * ll
                return 0

            remainlen = L - len(prefix)
            maxdigsum = remainlen * 9 + digsum
            digitcond = maxdigsum == n+1
            if maxdigsum == n or digitcond:
                ret = ''.join(prefix)
                if digitcond:
                    ret += "9"*(remaining_m-1) + "8" + \
                           "9"*(remainlen-remaining_m)
                else:
                    ret += "9"*remainlen
                return int(ret)

            mydiff8 = sum(
               count_perms(
                   self._seq_counts[i],
                   i,
               ) for i in self.nrange
            )
            if mydiff8 < remaining_m:
                use_one = True
                if len(prefix) and remainlen:
                    myseqs_before = self._gen_01_L_initial_sequences(
                         remainlen,
                         n - digsum
                    )

                    def _count_total(seqs, suffix_len):
                        ret = 0
                        for ll, s in enumerate(seqs):
                            if s:
                                ret += s * nCr(suffix_len, ll)
                        return ret
                    expected_before = _count_total(
                        myseqs_before, remainlen
                    )
                    if 0:
                        got = sum([self._seq_counts[i]*nCr(remainlen, i+1)
                                   for i in range(min([
                                       1+remainlen,
                                       len(self._seq_counts)]))])
                        compare(want=got,
                                have=expected_before,
                                blurb="_seq_counts")
                    top = remainlen
                    bottom = 1
                    while use_one and (top-bottom > 1):
                        remainlen_to_check = (top+bottom) >> 1
                        count_new_9s = remainlen - remainlen_to_check
                        new_digsum = digsum + count_new_9s * 9
                        new_remain_digsum = n - new_digsum
                        if new_remain_digsum <= 0:
                            top = remainlen_to_check-1
                        else:
                            myseqs_after =\
                                self._gen_01_L_initial_sequences(
                                    remainlen_to_check,
                                    new_remain_digsum,
                                )
                            expected_after = _count_total(
                                myseqs_after, remainlen_to_check)
                            expected = expected_before - expected_after
                            if expected < remaining_m:
                                remaining_m -= expected
                                descend_several_9s(count_new_9s)
                                print_('count_new_9s', count_new_9s)
                                use_one = False
                            else:
                                bottom = remainlen_to_check + 1
                                # print_('new bottom =', bottom, expected)
                if use_one:
                    remaining_m -= mydiff8
                    descend(9)
                count9 += 1
                # print_('count9 =', count9, 'non9 =', non9)
            else:
                non9 += 1
                for d in zrange:
                    mydiff = sum(
                        count_perms(len(self.zs[i][d]), i)
                        for i in self.nrange
                    )
                    if mydiff < remaining_m:
                        remaining_m -= mydiff
                    else:
                        descend(d)
                        break


def main():
    # top = int(sys.argv.pop(1)) if len(sys.argv) >= 2 else 10000
    top = 120
    mat = MatrixMod(10 ** 9 + 7)
    s = 0
    if 1:
        for nn in range(2, 28):
            for k in range(1, 11):
                n = nn
                m = k
                ret = mat.calc_f(n, m)
                good_ret = not_so_brute_calc_f(n, m)
                # good_ret = ret
                compare(ret, good_ret, "calc_f({},{})".format(n, m))

    for k in range(1, 10 + 1):
        n = k ** 3
        m = n * k
        print_('before', n, m)
        sys.stdout.flush()
        ret = mat.calc_f(n, m)
        print_('after', n, m, ret)
        sys.stdout.flush()
        good_ret = not_so_brute_calc_f(n, m)
        # good_ret = ret
        compare(ret, good_ret, "calc_f(k={};n={};m={})".format(k, n, m))
        sys.stdout.flush()
        s += ret
        sys.stdout.flush()
        if k == 3:
            compare(s, 7128, "S(3)")

    s %= mat.MOD
    compare(s, 32287064, "s[10]")
    for k in range(11, top+1):
        n = k ** 3
        m = n * k
        print_('before', k, n, m)
        sys.stdout.flush()
        ret = mat.calc_f(n, m)
        print_('after', n, m)
        sys.stdout.flush()
        # good_ret = not_so_brute_calc_f(n, m)
        # good_ret = ret
        # compare(ret, good_ret, "calc_f(k={};n={};m={})".format(k, n, m))
        # sys.stdout.flush()
        s += ret
        s %= mat.MOD
        print_('after s', k, s)
        sys.stdout.flush()

    print_("Final result = {}".format(s))
    return
    fibs = [1, 2]
    fibs_sum = 0
    for fibs_idx in range(2, 90 + 1):
        fibs_sum += mat.calc_s(fibs[0])
        fibs = [fibs[1], sum(fibs)]
    print_("result = {}".format(fibs_sum % mat.MOD))
    return


if __name__ == "__main__":
    main()
